@@ -307,8 +307,10 @@ block_cache_create(struct block_cache_conf *config, struct s3backer_store *inner
     if (config->cache_file != NULL) {
         if ((r = s3b_dcache_open(&priv->dcache, config, block_cache_dcache_load, priv, config->perform_flush)) != 0)
             goto fail10;
-        if (config->perform_flush && priv->num_dirties > 0)
-            (*config->log)(LOG_INFO, "%u dirty blocks in cache file `%s' will be recovered", priv->num_dirties, config->cache_file);
+        if (config->perform_flush && priv->num_dirties > 0) {
+            (*config->log)(LOG_INFO, "%u dirty blocks in cache file \"%s\" will be recovered",
+              priv->num_dirties, config->cache_file);
+        }
         priv->stats.initial_size = priv->num_cleans + priv->num_dirties;
     }
 
