@@ -1,19 +1,19 @@
 
 /*
  * s3backer - FUSE-based single file backing store via Amazon S3
- * 
- * Copyright 2008-2011 Archie L. Cobbs <archie@dellroad.org>
- * 
+ *
+ * Copyright 2008-2023 Archie L. Cobbs <archie.cobbs@gmail.com>
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
@@ -36,8 +36,8 @@
 
 #include "s3backer.h"
 #include "ec_protect.h"
-#include "block_part.h"
 #include "hash.h"
+#include "util.h"
 
 /*
  * Written block information caching.
@@ -46,21 +46,31 @@
  * by S3's "eventual consistency". We do this by:
  *
  *  (a) Enforcing a minimum delay between the completion of one PUT/DELETE
- *      of a block and the initiation of the next PUT/DELETE of the same block
- *  (b) Caching the MD5 checksum of every block written for some minimum time
- *      and verifying that data returned from subsequent GETs is correct.
+ *      of a block and the initiation of the next PUT/DELETE of the same block.
+ *      This is to allow the PUT/DELETE to be propagated across the S3 network.
+ *  (b) Caching the ETag (which is usually just the MD5 checksum) of every block
+ *      written for some minimum time and verifying that data returned from subsequent
+ *      GETs matches, allowing us to verify we are not reading back stale data.
+ *
+ * The theory here is that if you choose a value of X seconds large enough, and you
+ * assume all S3 reads will be up-to-date X seconds after they've been written, then
+ * using X seconds with (a) and (b) above guarantees consistent reads. However, this
+ * guarantee is only as good as that assumption about maximum convergence time X.
  *
  * These are the relevant configuration parameters:
  *
  *  min_write_delay
  *      Minimum time delay after a PUT/DELETE completes before the next PUT/DELETE
- *      can be initiated.
+ *      can be initiated (for the same block).
  *  cache_time
- *      How long after writing a block we'll remember its MD5 checksum. This
- *      must be at least as long as min_write_delay. Zero means infinity.
+ *      How long after writing a block we'll remember its ETag. This must be
+ *      at least as long as min_write_delay. This value determines the limit of
+ *      our ability to detect out-of-date reads. Zero means infinity.
  *  cache_size
- *      Maximum number of blocks we'll track at one time. When table
- *      is full, additional writes will block.
+ *      Maximum number of blocks we'll track at one time. When the table is full
+ *      and no WRITTEN blocks have exipred yet, additional writes will block.
+ *      Note if cache_time is zero (infinity), then this must be big enough to
+ *      contain ALL of the blocks, otherwise you will eventually deadlock.
  *
  * Blocks we are currently tracking can be in the following states:
  *
@@ -69,7 +79,7 @@
  *
  * CLEAN    initial state            No          No
  * WRITING  currently being written  Yes         No    timestamp == 0, u.data valid
- * WRITTEN  written and MD5 cached   Yes         Yes   timestamp != 0, u.md5 valid
+ * WRITTEN  written and ETag cached  Yes         Yes   timestamp != 0, u.etag valid
  *
  * The steady state for a block is CLEAN. WRITING means the block is currently
  * being sent; concurrent attempts to write will simply sleep until the first one
@@ -77,15 +87,15 @@
  * state will timeout (and the entry revert to CLEAN) after cache_time.
  *
  * If another attempt to write a block in the WRITTEN state occurs occurs before
- * min_write_delay has elapsed, the second attempt will sleep.
+ * min_write_delay has elapsed, the second attempt will sleep until it has.
  *
  * In the WRITING state, we have the data still so any reads are local. In the WRITTEN
- * state we don't have the data but we do know its MD5, so therefore we can verify what
- * comes back; if it doesn't verify, we retry as we would with any other error.
+ * state we don't have the data but we do know its ETag, so therefore we can verify what
+ * comes back from the read; if it doesn't verify, we retry as we would with any other error.
  *
  * There is a special case that occurs when we get an error while WRITING: in this case,
  * we don't know whether the block was successfully written or not, so we transition to
- * WRITTEN but with an all zeroes MD5 indicating "don't know".
+ * WRITTEN but with an all zeros ETag indicating "don't know".
  *
  * If we hit the 'cache_size' limit, we sleep a little while and then try again.
  *
@@ -103,12 +113,12 @@ struct block_info {
     uint64_t                timestamp;          // time PUT/DELETE completed (if WRITTEN)
     TAILQ_ENTRY(block_info) link;               // list entry link
     union {
-        const void      *data;                  // blocks actual content (if WRITING)
-        u_char          md5[MD5_DIGEST_LENGTH]; // block's content MD5 (if WRITTEN)
+        const void      *data;                  // block's actual content (if WRITING)
+        u_char          etag[MD5_DIGEST_LENGTH];// block's ETag (if WRITTEN)
     } u;
 };
 
-/* Internal state */
+// Internal state
 struct ec_protect_private {
     struct ec_protect_conf      *config;
     struct s3backer_store       *inner;
@@ -116,42 +126,37 @@ struct ec_protect_private {
     struct s3b_hash             *hashtable;
     u_int                       num_sleepers;   // count of sleeping threads
     TAILQ_HEAD(, block_info)    list;
+    block_list_func_t           *survey_callback;// non-zero survey is running and this is the callback
+    void                        *survey_arg;    // non-zero survey is running and this is the arg
     pthread_mutex_t             mutex;
     pthread_cond_t              space_cond;     // signaled when cache space available
     pthread_cond_t              sleepers_cond;  // signaled when no more threads are sleeping
     pthread_cond_t              never_cond;     // never signaled; used for sleeping only
 };
 
-/* Callback info */
-struct cbinfo {
-    block_list_func_t           *callback;
-    void                        *arg;
-};
-
-/* s3backer_store functions */
+// s3backer_store functions
 static int ec_protect_create_threads(struct s3backer_store *s3b);
 static int ec_protect_meta_data(struct s3backer_store *s3b, off_t *file_sizep, u_int *block_sizep);
 static int ec_protect_set_mount_token(struct s3backer_store *s3b, int32_t *old_valuep, int32_t new_value);
 static int ec_protect_read_block(struct s3backer_store *s3b, s3b_block_t block_num, void *dest,
-  u_char *actual_md5, const u_char *expect_md5, int strict);
-static int ec_protect_write_block(struct s3backer_store *s3b, s3b_block_t block_num, const void *src, u_char *md5,
+  u_char *actual_etag, const u_char *expect_etag, int strict);
+static int ec_protect_write_block(struct s3backer_store *s3b, s3b_block_t block_num, const void *src, u_char *etag,
   check_cancel_t *check_cancel, void *check_cancel_arg);
-static int ec_protect_read_block_part(struct s3backer_store *s3b, s3b_block_t block_num, u_int off, u_int len, void *dest);
-static int ec_protect_write_block_part(struct s3backer_store *s3b, s3b_block_t block_num, u_int off, u_int len, const void *src);
-static int ec_protect_flush(struct s3backer_store *s3b);
+static int ec_protect_flush_blocks(struct s3backer_store *s3b, const s3b_block_t *block_nums, u_int num_blocks, long timeout);
+static int ec_protect_shutdown(struct s3backer_store *s3b);
 static void ec_protect_destroy(struct s3backer_store *s3b);
 
-/* Misc */
+// Misc
 static uint64_t ec_protect_sleep_until(struct ec_protect_private *priv, pthread_cond_t *cond, uint64_t wake_time_millis);
 static void ec_protect_scrub_expired_writtens(struct ec_protect_private *priv, uint64_t current_time);
 static uint64_t ec_protect_get_time(void);
-static int ec_protect_list_blocks(struct s3backer_store *s3b, block_list_func_t *callback, void *arg);
-static void ec_protect_dirty_callback(void *arg, void *value);
-static void ec_protect_free_one(void *arg, void *value);
+static int ec_protect_survey_non_zero(struct s3backer_store *s3b, block_list_func_t *callback, void *arg);
+static s3b_hash_visit_t ec_protect_append_block_list;
+static s3b_hash_visit_t ec_protect_free_one;
 
-/* Invariants checking */
+// Invariants checking
 #ifndef NDEBUG
-static void ec_protect_check_one(void *arg, void *value);
+static s3b_hash_visit_t ec_protect_check_one;
 static void ec_protect_check_invariants(struct ec_protect_private *priv);
 
 #define EC_PROTECT_CHECK_INVARIANTS(priv)     ec_protect_check_invariants(priv)
@@ -159,11 +164,11 @@ static void ec_protect_check_invariants(struct ec_protect_private *priv);
 #define EC_PROTECT_CHECK_INVARIANTS(priv)     do { } while (0)
 #endif
 
-/* Special all-zeroes MD5 value signifying a zeroed block */
-static const u_char zero_md5[MD5_DIGEST_LENGTH];
+// Special all-zeros MD5 value signifying a zeroed block
+static const u_char zero_etag[MD5_DIGEST_LENGTH];
 
-/* Special all-onew MD5 value signifying a just-written block whose content is unknown */
-static u_char unknown_md5[MD5_DIGEST_LENGTH];
+// Special all-ones MD5 value signifying a just-written block whose content is unknown
+static u_char unknown_etag[MD5_DIGEST_LENGTH];
 
 /*
  * Constructor
@@ -177,7 +182,7 @@ ec_protect_create(struct ec_protect_conf *config, struct s3backer_store *inner)
     struct ec_protect_private *priv;
     int r;
 
-    /* Initialize structures */
+    // Initialize structures
     if ((s3b = calloc(1, sizeof(*s3b))) == NULL) {
         r = errno;
         (*config->log)(LOG_ERR, "calloc(): %s", strerror(r));
@@ -188,10 +193,10 @@ ec_protect_create(struct ec_protect_conf *config, struct s3backer_store *inner)
     s3b->set_mount_token = ec_protect_set_mount_token;
     s3b->read_block = ec_protect_read_block;
     s3b->write_block = ec_protect_write_block;
-    s3b->read_block_part = ec_protect_read_block_part;
-    s3b->write_block_part = ec_protect_write_block_part;
-    s3b->list_blocks = ec_protect_list_blocks;
-    s3b->flush = ec_protect_flush;
+    s3b->bulk_zero = generic_bulk_zero;
+    s3b->flush_blocks = ec_protect_flush_blocks;
+    s3b->survey_non_zero = ec_protect_survey_non_zero;
+    s3b->shutdown = ec_protect_shutdown;
     s3b->destroy = ec_protect_destroy;
     if ((priv = calloc(1, sizeof(*priv))) == NULL) {
         r = errno;
@@ -212,9 +217,9 @@ ec_protect_create(struct ec_protect_conf *config, struct s3backer_store *inner)
     if ((r = s3b_hash_create(&priv->hashtable, config->cache_size)) != 0)
         goto fail6;
     s3b->data = priv;
-    memset(unknown_md5, 0xff, sizeof(unknown_md5));
+    memset(unknown_etag, 0xff, sizeof(unknown_etag));
 
-    /* Done */
+    // Done
     EC_PROTECT_CHECK_INVARIANTS(priv);
     return s3b;
 
@@ -261,21 +266,31 @@ ec_protect_set_mount_token(struct s3backer_store *s3b, int32_t *old_valuep, int3
 }
 
 static int
-ec_protect_flush(struct s3backer_store *const s3b)
+ec_protect_flush_blocks(struct s3backer_store *s3b, const s3b_block_t *block_nums, u_int num_blocks, long timeout)
 {
     struct ec_protect_private *const priv = s3b->data;
 
-    /* Grab lock and sanity check */
+    return (*priv->inner->flush_blocks)(priv->inner, block_nums, num_blocks, timeout);
+}
+
+static int
+ec_protect_shutdown(struct s3backer_store *const s3b)
+{
+    struct ec_protect_private *const priv = s3b->data;
+
+    // Grab lock and sanity check
     pthread_mutex_lock(&priv->mutex);
     EC_PROTECT_CHECK_INVARIANTS(priv);
 
-    /* Wait for all sleeping writers to finish */
+    // Wait for all sleeping writers to finish
     while (priv->num_sleepers > 0)
         pthread_cond_wait(&priv->sleepers_cond, &priv->mutex);
 
-    /* Release lock */
-    pthread_mutex_unlock(&priv->mutex);
-    return 0;
+    // Release lock
+    CHECK_RETURN(pthread_mutex_unlock(&priv->mutex));
+
+    // Propagate to lower layer
+    return (*priv->inner->shutdown)(priv->inner);
 }
 
 static void
@@ -283,18 +298,16 @@ ec_protect_destroy(struct s3backer_store *const s3b)
 {
     struct ec_protect_private *const priv = s3b->data;
 
-    /* Grab lock and sanity check */
+    // Grab lock and sanity check
     pthread_mutex_lock(&priv->mutex);
     EC_PROTECT_CHECK_INVARIANTS(priv);
+    assert(priv->num_sleepers == 0);
 
-    /* Wait for all sleeping writers to finish */
-    while (priv->num_sleepers > 0)
-        pthread_cond_wait(&priv->sleepers_cond, &priv->mutex);
-
-    /* Destroy inner store */
+    // Destroy inner store
     (*priv->inner->destroy)(priv->inner);
 
-    /* Free structures */
+    // Free structures
+    CHECK_RETURN(pthread_mutex_unlock(&priv->mutex));
     pthread_mutex_destroy(&priv->mutex);
     pthread_cond_destroy(&priv->space_cond);
     pthread_cond_destroy(&priv->sleepers_cond);
@@ -313,7 +326,7 @@ ec_protect_get_stats(struct s3backer_store *s3b, struct ec_protect_stats *stats)
     pthread_mutex_lock(&priv->mutex);
     memcpy(stats, &priv->stats, sizeof(*stats));
     stats->current_cache_size = s3b_hash_size(priv->hashtable);
-    pthread_mutex_unlock(&priv->mutex);
+    CHECK_RETURN(pthread_mutex_unlock(&priv->mutex));
 }
 
 void
@@ -323,67 +336,103 @@ ec_protect_clear_stats(struct s3backer_store *s3b)
 
     pthread_mutex_lock(&priv->mutex);
     memset(&priv->stats, 0, sizeof(priv->stats));
-    pthread_mutex_unlock(&priv->mutex);
+    CHECK_RETURN(pthread_mutex_unlock(&priv->mutex));
 }
 
 static int
-ec_protect_list_blocks(struct s3backer_store *s3b, block_list_func_t *callback, void *arg)
+ec_protect_survey_non_zero(struct s3backer_store *s3b, block_list_func_t *callback, void *arg)
 {
     struct ec_protect_private *const priv = s3b->data;
-    struct cbinfo cbinfo;
+    struct block_list list;
     int r;
 
-    if ((r = (*priv->inner->list_blocks)(priv->inner, callback, arg)) != 0)
-        return r;
-    cbinfo.callback = callback;
-    cbinfo.arg = arg;
+    // Lock mutex
     pthread_mutex_lock(&priv->mutex);
-    s3b_hash_foreach(priv->hashtable, ec_protect_dirty_callback, &cbinfo);
-    pthread_mutex_unlock(&priv->mutex);
-    return 0;
+    assert(priv->survey_callback == NULL);
+
+    // Record survey in progress
+    priv->survey_callback = callback;
+    priv->survey_arg = arg;
+
+    // Inventory all blocks currently in the cache; we don't bother trying to discern the zero blocks
+    block_list_init(&list);
+    if ((r = s3b_hash_foreach(priv->hashtable, ec_protect_append_block_list, &list)) != 0)
+        goto done;
+
+    // Unlock mutex
+    CHECK_RETURN(pthread_mutex_unlock(&priv->mutex));
+
+    // Report all blocks inventoried above
+    (*callback)(arg, list.blocks, list.num_blocks);
+    block_list_free(&list);
+
+    // Invoke lower layer
+    r = (*priv->inner->survey_non_zero)(priv->inner, callback, arg);
+
+    // Lock mutex
+    pthread_mutex_lock(&priv->mutex);
+
+    // Finish up
+    assert(priv->survey_callback != NULL);
+    priv->survey_callback = NULL;
+    priv->survey_arg = NULL;
+
+done:
+    // Done
+    CHECK_RETURN(pthread_mutex_unlock(&priv->mutex));
+    return r;
+}
+
+static int
+ec_protect_append_block_list(void *arg, void *value)
+{
+    struct block_info *const entry = value;
+    struct block_list *const list = arg;
+
+    return block_list_append(list, entry->block_num);
 }
 
 static int
 ec_protect_read_block(struct s3backer_store *const s3b, s3b_block_t block_num, void *dest,
-  u_char *actual_md5, const u_char *expect_md5, int strict)
+  u_char *actual_etag, const u_char *expect_etag, int strict)
 {
     struct ec_protect_private *const priv = s3b->data;
     struct ec_protect_conf *const config = priv->config;
-    u_char md5[MD5_DIGEST_LENGTH];
+    u_char etag[MD5_DIGEST_LENGTH];
     struct block_info *binfo;
 
-    /* Sanity check */
+    // Sanity check
     if (config->block_size == 0)
         return EINVAL;
 
-    /* Grab lock and sanity check */
+    // Grab lock and sanity check
     pthread_mutex_lock(&priv->mutex);
     EC_PROTECT_CHECK_INVARIANTS(priv);
 
 again:
-    /* Scrub the list of WRITTENs */
+    // Scrub the list of WRITTENs
     ec_protect_scrub_expired_writtens(priv, ec_protect_get_time());
 
-    /* Find info for this block */
+    // Find info for this block
     if ((binfo = s3b_hash_get(priv->hashtable, block_num)) != NULL) {
 
-        /* In WRITING state: we have the data already! */
+        // In WRITING state: we have the data already!
         if (binfo->timestamp == 0) {
             if (binfo->u.data == NULL)
                 memset(dest, 0, config->block_size);
             else
                 memcpy(dest, binfo->u.data, config->block_size);
-            if (actual_md5 != NULL)
-                memset(actual_md5, 0, MD5_DIGEST_LENGTH);           // we don't know it yet!
+            if (actual_etag != NULL)
+                memset(actual_etag, 0, MD5_DIGEST_LENGTH);          // we don't know it yet!
             priv->stats.cache_data_hits++;
-            pthread_mutex_unlock(&priv->mutex);
+            CHECK_RETURN(pthread_mutex_unlock(&priv->mutex));
             return 0;
         }
 
-        /* In WRITTEN state: special case: unknown MD5. Wait for settle time, then try again */
-        if (memcmp(binfo->u.md5, unknown_md5, MD5_DIGEST_LENGTH) == 0) {
+        // In WRITTEN state: special case: unknown ETag. Wait for settle time, then try again
+        if (memcmp(binfo->u.etag, unknown_etag, MD5_DIGEST_LENGTH) == 0) {
 
-            /* Have we waited long enough already? If so, reset block and try again */
+            // Have we waited long enough already? If so, reset block and try again
             if (ec_protect_get_time() >= binfo->timestamp + config->min_write_delay) {
                 TAILQ_REMOVE(&priv->list, binfo, link);
                 s3b_hash_remove(priv->hashtable, binfo->block_num);
@@ -391,93 +440,97 @@ again:
                 goto again;
             }
 
-            /* Sleep to allow previous failed write to resolve, and then try again */
+            // Sleep to allow previous failed write to resolve, and then try again
             ec_protect_sleep_until(priv, NULL, binfo->timestamp + config->min_write_delay);
             goto again;
         }
 
-        /* In WRITTEN state: special case: zero block */
-        if (memcmp(binfo->u.md5, zero_md5, MD5_DIGEST_LENGTH) == 0) {
-            if (expect_md5 != NULL && strict && memcmp(expect_md5, zero_md5, MD5_DIGEST_LENGTH) != 0)
-                (*config->log)(LOG_ERR, "ec_protect_read_block(): impossible expected MD5?");
+        // In WRITTEN state: special case: zero block
+        if (memcmp(binfo->u.etag, zero_etag, MD5_DIGEST_LENGTH) == 0) {
+            if (expect_etag != NULL && strict && memcmp(expect_etag, zero_etag, MD5_DIGEST_LENGTH) != 0)
+                (*config->log)(LOG_ERR, "ec_protect_read_block(): impossible expected ETag?");
             memset(dest, 0, config->block_size);
-            if (actual_md5 != NULL)
-                memset(actual_md5, 0, MD5_DIGEST_LENGTH);
+            if (actual_etag != NULL)
+                memset(actual_etag, 0, MD5_DIGEST_LENGTH);
             priv->stats.cache_data_hits++;
-            pthread_mutex_unlock(&priv->mutex);
+            CHECK_RETURN(pthread_mutex_unlock(&priv->mutex));
             return 0;
         }
 
-        /* In WRITTEN state: we know the expected MD5 */
-        memcpy(md5, binfo->u.md5, MD5_DIGEST_LENGTH);
-        if (expect_md5 != NULL && strict && memcmp(md5, expect_md5, MD5_DIGEST_LENGTH) != 0)
-            (*config->log)(LOG_ERR, "ec_protect_read_block(): impossible expected MD5?");
-        expect_md5 = md5;
+        // In WRITTEN state: we know the expected ETag
+        memcpy(etag, binfo->u.etag, MD5_DIGEST_LENGTH);
+        if (expect_etag != NULL && strict && memcmp(etag, expect_etag, MD5_DIGEST_LENGTH) != 0)
+            (*config->log)(LOG_ERR, "ec_protect_read_block(): impossible expected ETag?");
+        expect_etag = etag;
         strict = 1;
     }
 
-    /* Release lock */
-    pthread_mutex_unlock(&priv->mutex);
+    // Release lock
+    CHECK_RETURN(pthread_mutex_unlock(&priv->mutex));
 
-    /* Read block normally */
-    return (*priv->inner->read_block)(priv->inner, block_num, dest, actual_md5, expect_md5, strict);
+    // Read block normally
+    return (*priv->inner->read_block)(priv->inner, block_num, dest, actual_etag, expect_etag, strict);
 }
 
 static int
-ec_protect_write_block(struct s3backer_store *const s3b, s3b_block_t block_num, const void *src, u_char *caller_md5,
+ec_protect_write_block(struct s3backer_store *const s3b, s3b_block_t block_num, const void *src, u_char *caller_etag,
   check_cancel_t *check_cancel, void *check_cancel_arg)
 {
     struct ec_protect_private *const priv = s3b->data;
     struct ec_protect_conf *const config = priv->config;
-    u_char md5[MD5_DIGEST_LENGTH];
+    u_char etag[MD5_DIGEST_LENGTH];
     struct block_info *binfo;
     uint64_t current_time;
     uint64_t delay;
     int r;
 
-    /* Sanity check */
+    // Sanity check
     if (config->block_size == 0)
         return EINVAL;
 
-    /* Grab lock */
+    // Grab lock
     pthread_mutex_lock(&priv->mutex);
 
+    // Conservatively disqualify any non-zero block as being zero in any ongoing non-zero survey
+    if (src != NULL && priv->survey_callback != NULL)
+        (*priv->survey_callback)(priv->survey_arg, &block_num, 1);
+
 again:
-    /* Sanity check */
+    // Sanity check
     EC_PROTECT_CHECK_INVARIANTS(priv);
 
-    /* Scrub the list of WRITTENs */
+    // Scrub the list of WRITTENs
     current_time = ec_protect_get_time();
     ec_protect_scrub_expired_writtens(priv, current_time);
 
-    /* Find info for this block */
+    // Find info for this block
     binfo = s3b_hash_get(priv->hashtable, block_num);
 
-    /* CLEAN case: add new entry in state WRITING and write the block */
+    // CLEAN case: add new entry in state WRITING and write the block
     if (binfo == NULL) {
 
-        /* If we have reached max cache capacity, wait until there's more room */
+        // If we have reached max cache capacity, wait until there's more room
         if (s3b_hash_size(priv->hashtable) >= config->cache_size) {
 
-            /* Report deadlock situation */
+            // Report deadlock situation
             if (config->cache_time == 0)
                 (*config->log)(LOG_ERR, "md5 cache is full, but timeout is infinite: you have write deadlock!");
 
-            /* Sleep until space becomes available */
+            // Sleep until space becomes available
             if ((binfo = TAILQ_FIRST(&priv->list)) != NULL && config->cache_time > 0)
                 delay = ec_protect_sleep_until(priv, &priv->space_cond, binfo->timestamp + config->cache_time);
             else
-                delay = ec_protect_sleep_until(priv, &priv->space_cond, 0);         /* sleep indefinitely... */
+                delay = ec_protect_sleep_until(priv, &priv->space_cond, 0);         // sleep indefinitely...
             priv->stats.cache_full_delay += delay;
             goto again;
         }
 
-        /* Create new entry in WRITING state */
+        // Create new entry in WRITING state
         if ((binfo = calloc(1, sizeof(*binfo))) == NULL) {
             r = errno;
             (*config->log)(LOG_ERR, "can't alloc new MD5 cache entry: %s", strerror(r));
             priv->stats.out_of_memory_errors++;
-            pthread_mutex_unlock(&priv->mutex);
+            CHECK_RETURN(pthread_mutex_unlock(&priv->mutex));
             return r;
         }
         binfo->block_num = block_num;
@@ -485,9 +538,9 @@ again:
         s3b_hash_put_new(priv->hashtable, binfo);
 
 writeit:
-        /* Write the block */
-        pthread_mutex_unlock(&priv->mutex);
-        r = (*priv->inner->write_block)(priv->inner, block_num, src, md5, check_cancel, check_cancel_arg);
+        // Write the block
+        CHECK_RETURN(pthread_mutex_unlock(&priv->mutex));
+        r = (*priv->inner->write_block)(priv->inner, block_num, src, etag, check_cancel, check_cancel_arg);
         pthread_mutex_lock(&priv->mutex);
         EC_PROTECT_CHECK_INVARIANTS(priv);
 
@@ -502,17 +555,17 @@ writeit:
          * Move to state WRITTEN.
          *
          * If there was an error, we can't assume we know whether the write succeeded or not,
-         * so mark the block as WRITTEN but with a special MD5 value meaning "unknown".
+         * so mark the block as WRITTEN but with a special ETag value meaning "unknown".
          * We have to wait for min_write_delay before trying to read the block again.
          */
         binfo->timestamp = ec_protect_get_time();
-        memcpy(binfo->u.md5, r == 0 ? md5 : unknown_md5, MD5_DIGEST_LENGTH);
+        memcpy(binfo->u.etag, r == 0 ? etag : unknown_etag, MD5_DIGEST_LENGTH);
         TAILQ_INSERT_TAIL(&priv->list, binfo, link);
-        pthread_mutex_unlock(&priv->mutex);
+        CHECK_RETURN(pthread_mutex_unlock(&priv->mutex));
 
-        /* Copy expected MD5 for caller */
-        if (r == 0 && caller_md5 != NULL)
-            memcpy(caller_md5, md5, MD5_DIGEST_LENGTH);
+        // Copy expected ETag for caller
+        if (r == 0 && caller_etag != NULL)
+            memcpy(caller_etag, etag, MD5_DIGEST_LENGTH);
         return r;
     }
 
@@ -546,24 +599,6 @@ writeit:
     goto writeit;
 }
 
-static int
-ec_protect_read_block_part(struct s3backer_store *s3b, s3b_block_t block_num, u_int off, u_int len, void *dest)
-{
-    struct ec_protect_private *const priv = s3b->data;
-    struct ec_protect_conf *const config = priv->config;
-
-    return block_part_read_block_part(s3b, block_num, config->block_size, off, len, dest);
-}
-
-static int
-ec_protect_write_block_part(struct s3backer_store *s3b, s3b_block_t block_num, u_int off, u_int len, const void *src)
-{
-    struct ec_protect_private *const priv = s3b->data;
-    struct ec_protect_conf *const config = priv->config;
-
-    return block_part_write_block_part(s3b, block_num, config->block_size, off, len, src);
-}
-
 /*
  * Return current time in milliseconds.
  */
@@ -644,32 +679,23 @@ ec_protect_sleep_until(struct ec_protect_private *priv, pthread_cond_t *cond, ui
     return time_after - time_before;
 }
 
-static void
+static int
 ec_protect_free_one(void *arg, void *value)
 {
     free(value);
-}
-
-static void
-ec_protect_dirty_callback(void *arg, void *value)
-{
-    struct cbinfo *const cbinfo = arg;
-    struct block_info *const binfo = value;
-
-    if (binfo->timestamp == 0 ? binfo->u.data != NULL : memcmp(binfo->u.md5, zero_md5, MD5_DIGEST_LENGTH) != 0)
-        (*cbinfo->callback)(cbinfo->arg, binfo->block_num);
+    return 0;
 }
 
 #ifndef NDEBUG
 
-/* Accounting structure */
+// Accounting structure
 struct check_info {
     u_int   num_in_list;
     u_int   written;
     u_int   writing;
 };
 
-static void
+static int
 ec_protect_check_one(void *arg, void *value)
 {
     struct block_info *const binfo = value;
@@ -679,6 +705,7 @@ ec_protect_check_one(void *arg, void *value)
         info->writing++;
     else
         info->written++;
+    return 0;
 }
 
 static void
