@@ -66,12 +66,11 @@ struct stat_file {
  ****************************************************************************/
 
 // FUSE functions
-static void *fuse_op_init(struct fuse_conn_info *conn);
+static void *fuse_op_init(struct fuse_conn_info *conn, struct fuse_config *cfg);
 static void fuse_op_destroy(void *data);
-static int fuse_op_getattr(const char *path, struct stat *st);
-static int fuse_op_fgetattr(const char *path, struct stat *st, struct fuse_file_info *);
+static int fuse_op_getattr(const char *path, struct stat *st, struct fuse_file_info *fi);
 static int fuse_op_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
-    off_t offset, struct fuse_file_info *fi);
+    off_t offset, struct fuse_file_info *fi, enum fuse_readdir_flags flags);
 static int fuse_op_open(const char *path, struct fuse_file_info *fi);
 static int fuse_op_release(const char *path, struct fuse_file_info *fi);
 static int fuse_op_read(const char *path, char *buf, size_t size, off_t offset,
@@ -79,13 +78,11 @@ static int fuse_op_read(const char *path, char *buf, size_t size, off_t offset,
 static int fuse_op_write(const char *path, const char *buf, size_t size, off_t offset,
     struct fuse_file_info *fi);
 static int fuse_op_statfs(const char *path, struct statvfs *st);
-static int fuse_op_truncate(const char *path, off_t size);
+static int fuse_op_truncate(const char *path, off_t size, struct fuse_file_info *fi);
 static int fuse_op_flush(const char *path, struct fuse_file_info *fi);
 static int fuse_op_fsync(const char *path, int isdatasync, struct fuse_file_info *fi);
 static int fuse_op_unlink(const char *path);
-#if FUSE_FALLOCATE
 static int fuse_op_fallocate(const char *path, int mode, off_t offset, off_t len, struct fuse_file_info *fi);
-#endif
 
 // Attribute functions
 static void fuse_op_getattr_file(struct fuse_ops_private *priv, struct stat *st);
@@ -95,6 +92,8 @@ static void fuse_op_getattr_stats(struct fuse_ops_private *priv, struct stat_fil
 static struct stat_file *fuse_op_stats_create(struct fuse_ops_private *priv);
 static void fuse_op_stats_destroy(struct stat_file *sfile);
 static printer_t fuse_op_stats_printer;
+static printer_t stats_mirror_printer;
+static void *stats_mirror_thread(void *arg);
 
 /****************************************************************************
  *                          VARIABLE DEFINITIONS                            *
@@ -105,7 +104,6 @@ const struct fuse_operations s3backer_fuse_ops = {
     .init       = fuse_op_init,
     .destroy    = fuse_op_destroy,
     .getattr    = fuse_op_getattr,
-    .fgetattr   = fuse_op_fgetattr,
     .readdir    = fuse_op_readdir,
     .open       = fuse_op_open,
     .read       = fuse_op_read,
@@ -116,9 +114,7 @@ const struct fuse_operations s3backer_fuse_ops = {
     .fsync      = fuse_op_fsync,
     .release    = fuse_op_release,
     .unlink     = fuse_op_unlink,
-#if FUSE_FALLOCATE
     .fallocate  = fuse_op_fallocate,
-#endif
 };
 
 // Configuration and underlying s3backer_store
@@ -162,10 +158,12 @@ fuse_ops_create(struct fuse_ops_conf *config0, struct s3backer_store *s3b)
 void
 fuse_ops_destroy(void)
 {
-    struct fuse_ops_private *const priv = the_priv;
-
-    block_part_destroy(&priv->block_part);
-    fuse_op_destroy(priv);
+    if (the_priv == NULL)
+        return;
+    block_part_destroy(&the_priv->block_part);
+    fuse_op_destroy(the_priv);
+    free(the_priv);
+    the_priv = NULL;
 }
 
 /****************************************************************************
@@ -173,7 +171,7 @@ fuse_ops_destroy(void)
  ****************************************************************************/
 
 static void *
-fuse_op_init(struct fuse_conn_info *conn)
+fuse_op_init(struct fuse_conn_info *conn, struct fuse_config *cfg)
 {
     struct s3b_config *const s3bconf = config->s3bconf;
     struct fuse_ops_private *const priv = the_priv;
@@ -194,12 +192,28 @@ fuse_op_init(struct fuse_conn_info *conn)
     // Apply process tweaks
     apply_process_tweaks();
 
-    // Startup background threads now that we have fork()'d
-    if ((r = (*priv->s3b->create_threads)(priv->s3b)) != 0) {
-        (*config->log)(LOG_ERR, "fuse_op_init(): can't create threads: %s", strerror(errno));
+    // Start up stats mirror thread
+    priv->stats_mirror_state = STATS_MIRROR_INITIAL;
+    if (config->stats_mirror_path != NULL
+      && (r = pthread_create(&priv->stats_mirror_thread, NULL, stats_mirror_thread, priv)) != 0) {
+        (*config->log)(LOG_ERR, "failed to create stats mirror thread: %s", strerror(r));
         return NULL;
     }
 
+    // Startup other background threads
+    if ((r = (*priv->s3b->create_threads)(priv->s3b)) != 0) {
+        (*config->log)(LOG_ERR, "fuse_op_init(): can't create threads: %s", strerror(errno));
+        if (config->stats_mirror_path != NULL) {
+            (void)pthread_cancel(priv->stats_mirror_thread);
+            (void)pthread_join(priv->stats_mirror_thread, NULL);
+        }
+        return NULL;
+    }
+
+    // Allow stats mirror thread to start writing
+    if (config->stats_mirror_path != NULL)
+        priv->stats_mirror_state = STATS_MIRROR_RUNNING;
+
     // Done
     (*config->log)(LOG_INFO, "mounting %s", s3bconf->mount);
     return priv;
@@ -218,13 +232,21 @@ fuse_op_destroy(void *data)
         return;
     (*config->log)(LOG_INFO, "unmount %s: initiated", s3bconf->mount);
 
-    // Shutdown (flush dirty data)
+    // Shutdown stats mirror thread, if any
+    if (priv->stats_mirror_state == STATS_MIRROR_RUNNING) {
+        priv->stats_mirror_state = STATS_MIRROR_SHUTDOWN;
+        (void)pthread_cancel(priv->stats_mirror_thread);
+        (void)pthread_join(priv->stats_mirror_thread, NULL);
+        priv->stats_mirror_state = STATS_MIRROR_INITIAL;
+    }
+
+    // Shutdown S3 statck (flush dirty data)
     (*config->log)(LOG_INFO, "unmount %s: shutting down filesystem", s3bconf->mount);
     if ((r = (*s3b->shutdown)(s3b)) != 0)
         (*config->log)(LOG_ERR, "unmount %s: filesystem shutdown failed: %s", s3bconf->mount, strerror(r));
 
-    // Clear mount token
-    if (!config->read_only) {
+    // Clear mount token (non-shared disk mode only)
+    if (!config->read_only && !s3bconf->shared_disk_mode) {
         (*config->log)(LOG_INFO, "unmount %s: clearing mount token", s3bconf->mount);
         if ((r = (*s3b->set_mount_token)(s3b, NULL, 0)) != 0)
             (*config->log)(LOG_ERR, "unmount %s: clearing mount token failed: %s", s3bconf->mount, strerror(r));
@@ -232,12 +254,12 @@ fuse_op_destroy(void *data)
 
     // Destroy
     (*s3b->destroy)(s3b);
+    priv->s3b = NULL;
     (*config->log)(LOG_INFO, "unmount %s: completed", s3bconf->mount);
-    free(priv);
 }
 
 static int
-fuse_op_getattr(const char *path, struct stat *st)
+fuse_op_getattr(const char *path, struct stat *st, struct fuse_file_info *fi)
 {
     struct fuse_ops_private *const priv = (struct fuse_ops_private *)fuse_get_context()->private_data;
 
@@ -273,20 +295,6 @@ fuse_op_getattr(const char *path, struct stat *st)
     return -ENOENT;
 }
 
-static int
-fuse_op_fgetattr(const char *path, struct stat *st, struct fuse_file_info *fi)
-{
-    struct fuse_ops_private *const priv = (struct fuse_ops_private *)fuse_get_context()->private_data;
-
-    if (fi->fh != 0) {
-        struct stat_file *const sfile = (struct stat_file *)(uintptr_t)fi->fh;
-
-        fuse_op_getattr_stats(priv, sfile, st);
-    } else
-        fuse_op_getattr_file(priv, st);
-    return 0;
-}
-
 static void
 fuse_op_getattr_file(struct fuse_ops_private *priv, struct stat *st)
 {
@@ -321,7 +329,7 @@ fuse_op_getattr_stats(struct fuse_ops_private *priv, struct stat_file *sfile, st
 
 static int
 fuse_op_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
-    off_t offset, struct fuse_file_info *fi)
+    off_t offset, struct fuse_file_info *fi, enum fuse_readdir_flags flags)
 {
     struct fuse_ops_private *const priv = (struct fuse_ops_private *)fuse_get_context()->private_data;
 
@@ -329,15 +337,15 @@ fuse_op_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
     (void)fi;
     if (strcmp(path, "/") != 0)
         return -ENOENT;
-    if (filler(buf, ".", NULL, 0) != 0)
+    if (filler(buf, ".", NULL, 0, 0) != 0)
         return -ENOMEM;
-    if (filler(buf, "..", NULL, 0) != 0)
+    if (filler(buf, "..", NULL, 0, 0) != 0)
         return -ENOMEM;
     if (priv != NULL) {
-        if (filler(buf, config->filename, NULL, 0) != 0)
+        if (filler(buf, config->filename, NULL, 0, 0) != 0)
             return -ENOMEM;
         if (config->print_stats != NULL && config->stats_filename != NULL) {
-            if (filler(buf, config->stats_filename, NULL, 0) != 0)
+            if (filler(buf, config->stats_filename, NULL, 0, 0) != 0)
                 return -ENOMEM;
         }
     }
@@ -499,7 +507,7 @@ fuse_op_statfs(const char *path, struct statvfs *st)
 }
 
 static int
-fuse_op_truncate(const char *path, off_t size)
+fuse_op_truncate(const char *path, off_t size, struct fuse_file_info *fi)
 {
     return 0;
 }
@@ -544,7 +552,6 @@ fuse_op_unlink(const char *path)
 }
 
 
-#if FUSE_FALLOCATE
 static int
 fuse_op_fallocate(const char *path, int mode, off_t offset, off_t len, struct fuse_file_info *fi)
 {
@@ -586,7 +593,6 @@ fuse_op_fallocate(const char *path, int mode, off_t offset, off_t len, struct fu
     priv->file_mtime = time(NULL);
     return 0;
 }
-#endif
 
 /****************************************************************************
  *                    OTHER INTERNAL FUNCTIONS                              *
@@ -650,3 +656,84 @@ again:
     goto again;
 }
 
+// Stats mirror thread
+static void *
+stats_mirror_thread(void *arg)
+{
+    struct fuse_ops_private *const priv = the_priv;
+    int file_was_there = 0;
+
+    while (1) {
+
+        // Time to leave?
+        if (priv->stats_mirror_state == STATS_MIRROR_SHUTDOWN)
+            break;
+
+        // If system is up & running, update the stats file
+        if (priv->stats_mirror_state == STATS_MIRROR_RUNNING) {
+            struct stat sb;
+            char *temp_path;
+            FILE *fp;
+
+            // If file existed before but was since deleted, reset stats
+            if (file_was_there && stat(config->stats_mirror_path, &sb) == -1) {
+                if (errno != ENOENT) {
+                    (*config->log)(LOG_ERR, "error %s stats mirror file %s (mirroring stopped): %s",
+                      "accessing", config->stats_mirror_path, strerror(errno));
+                    break;
+                }
+                (*config->clear_stats)();
+                file_was_there = 0;
+            }
+
+            // Format temporary name string
+            if (asprintf(&temp_path, "%s.new", config->stats_mirror_path) == -1) {
+                (*config->log)(LOG_ERR, "error %s stats mirror file %s (mirroring stopped): %s",
+                  "updating", config->stats_mirror_path, strerror(errno));
+                break;
+            }
+
+            // Write into temporary file
+            if ((fp = fopen(temp_path, "w")) == NULL) {
+                (*config->log)(LOG_ERR, "error %s stats mirror file %s (mirroring stopped): %s",
+                  "creating", temp_path, strerror(errno));
+                free(temp_path);
+                break;
+            }
+            (*config->print_stats)(fp, stats_mirror_printer);
+            if (fclose(fp) == EOF) {
+                (*config->log)(LOG_ERR, "error %s stats mirror file %s (mirroring stopped): %s",
+                  "writing", config->stats_mirror_path, strerror(errno));
+                unlink(temp_path);
+                free(temp_path);
+                break;
+            }
+            if (rename(temp_path, config->stats_mirror_path) == -1) {
+                (*config->log)(LOG_ERR, "error %s stats mirror file %s (mirroring stopped): %s",
+                  "updating", config->stats_mirror_path, strerror(errno));
+                unlink(temp_path);
+                free(temp_path);
+                break;
+            }
+            free(temp_path);
+            file_was_there = 1;
+        }
+
+        // Sleep for a while
+        usleep(config->stats_mirror_interval * 1000);
+    }
+
+    // Done
+    return NULL;
+}
+
+static void
+stats_mirror_printer(void *prarg, const char *fmt, ...)
+{
+    FILE *const fp = prarg;
+    va_list args;
+
+    va_start(args, fmt);
+    vfprintf(fp, fmt, args);
+    va_end(args);
+}
