@@ -65,6 +65,7 @@ struct s3b_config {
     int                         test;
     int                         ssl;
     int                         nbd;
+    int                         shared_disk_mode;
     int                         no_auto_detect;
     int                         list_blocks;
     struct fuse_args            fuse_args;
@@ -75,8 +76,8 @@ struct s3b_config {
     const char                  *block_size_str;
     const char                  *password_file;
     const char                  *max_speed_str[2];
-    const char                  *compress_alg;
-    const char                  *compress_level;
+    char                        *compress_alg;
+    char                        *compress_level;
     int                         compress_flag;
     int                         encrypt;
 };
